;Revision 4 - [03:22 2013-09-03]
;Revision 3 - [01:33 2013-08-26]
;Revision 2 - [18:18 2013-08-24]
;Revision 1 - [03:09 2013-08-24]

NULL := ""
SizeConDebug := 0
SizeConErrorLevel := 0
SizeConLib_VERSION = 2.0.0

;Type sizes // http://msdn.microsoft.com/library/aa383751.aspx // EXAMPLE: SHORT is 2 bytes, etc..
sType := Object("SHORT", 2, "COORD", 4, "WORD", 2, "SMALL_RECT", 8, "DWORD", 4)

dec2hex(var) {
	
	OldFormat := A_FormatInteger
	SetFormat, Integer, Hex
	var += 0
	SetFormat, Integer, %OldFormat%
	return var
}

hex2dec(var) {
	
	OldFormat := A_FormatInteger
	SetFormat, Integer, D
	var += 0
	SetFormat, Integer, %OldFormat%
	return var
}

;Launch a console with a PID
newConsole() {
	
	run, %ComSpec% /k cd `%userprofile`% &cls,, , cPID
	WinWait, ahk_pid %cPID%
	return %cPID%
}

;AttachConsole() http://msdn.microsoft.com/library/ms681952
AttachConsole(cPID) {
	
	global SizeConErrorLevel
	cmdlaunched := DllCall("AttachConsole", "int", cPID, "Cdecl int")
	if errorlevel
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("AttachConsole",cPID) ;Failure
	}
	else
	{
		if (cmdlaunched = 1)
			return 1 ;Success
		else
		{
			return MsgError("AttachConsole",cPID) ;Failure
		}
	}
}

;FreeConsole() http://msdn.microsoft.com/library/ms683150
FreeConsole() {
	
	global SizeConErrorLevel
	cmdfreed := DllCall("FreeConsole", int)
	if errorlevel
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("FreeConsole") ;Failure
	}
	else
	{
		if (cmdfreed = 1)
			return 1 ;Success
		else
		{
			return MsgError("FreeConsole") ;Failure
		}
	}
}

;Get the console's window Handle
getConsoleHandle() {
	
	global SizeConErrorLevel
	hConsole := DllCall("GetConsoleWindow","UPtr") ;or WinGet, hConsole, ID, ahk_pid %cPID%
	if errorlevel
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("GetConsoleWindow") ;Failure
	}
	else
		return %hConsole% ;Success
}

;Get Console screen buffer handle, Forked from original by Lexikos
;http://www.autohotkey.com/board/topic/88510-ufcpp-the-unofficial-fullscreen-command-prompt-patcher/#entry561509
getStdoutHandle() {
	
	global SizeConErrorLevel
	stdout := FileOpen(DllCall("GetStdHandle", "UInt", -11, "Cdecl Ptr"), "h") ;or f := FileOpen("CONOUT$", "w"), h := f.__Handle
	hStdout := stdout.__Handle
	if errorlevel
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("GetStdHandle") ;Failure
	}
	else
		return %hStdout% ;Success
}

;Get Window's clientsize, modified From Lexikos ; http://www.autohotkey.com/forum/post-170475.html
getClientSize(hwnd, ByRef w, ByRef h) {
    global SizeConErrorLevel
	VarSetCapacity(rc, 16)
    DllCall("GetClientRect", "uint", hwnd, "uint", &rc)
    w := NumGet(rc, 8, "int")
    h := NumGet(rc, 12, "int")
	if errorlevel
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("GetClientRect",hwnd,w,h) ;Failure
	}
	else
		return 1 ;Success
}

;GetSystemMetrics() http://msdn.microsoft.com/library/ms724385
getClientOffset(ByRef TitleBarY, ByRef BorderX, ByRef BorderY, ByRef ScrollBarY:="") {
	
	TitleBarY := DllCall("GetSystemMetrics","Int",4,"Cdecl Int") ;SM_CYCAPTION
	BorderX := DllCall("GetSystemMetrics","Int",7,"Cdecl Int") ;SM_CXFIXEDFRAME
	BorderY := DllCall("GetSystemMetrics","Int",8,"Cdecl Int") ;SM_CYFIXEDFRAME
	ScrollBarY := DllCall("GetSystemMetrics","Int",2,"Cdecl Int") ;SM_CXVSCROLL
}

;Get BufferSize, GetConsoleScreenBufferInfo()
getConsoleSize(hStdout, ByRef bufferwidth, ByRef bufferheight) {
	
	global sType
	;typedef struct _CONSOLE_SCREEN_BUFFER_INFO {
	;	COORD      dwSize;
	;	COORD      dwCursorPosition;
	;	WORD       wAttributes;
	;	SMALL_RECT srWindow;
	;	COORD      dwMaximumWindowSize;
	; } CONSOLE_SCREEN_BUFFER_INFO;
	
	;CONSOLE_SCREEN_BUFFER_INFO cmdinfo;
	;GetConsoleScreenBufferInfo(hStdout,&cmdinfo);
	VarSetCapacity(struct,(sType.COORD*3)+sType.WORD+sType.SMALL_RECT,0)

	rvalue:=DllCall("GetConsoleScreenBufferInfo","UPtr",hStdout,"Ptr",&struct)
	derror:=ErrorLevel
	
	bufferwidth:=NumGet(&struct,"UShort")
	bufferheight:=NumGet(&struct,sType.SHORT,"UShort")
	
	if (derror) or (rvalue=0)
	{
		SizeConErrorLevel := derror
		return MsgError("getConsoleSize",hStdout) ;Failure
	}
	else
		return 0 ;Success
}

getConsoleWidth(hStdout) {
	
	global SizeConErrorLevel
	
	if (getConsoleSize(hStdout,bufferwidth,bufferheight))
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("getConsoleWidth",hStdout) ;Failure
	}
	else
		return %bufferwidth% ;Success
}

getConsoleHeight(hStdout) {
	
	global SizeConErrorLevel
	
	if (getConsoleSize(hStdout,bufferwidth,bufferheight))
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("getConsoleHeight",hStdout) ;Failure
	}
	else
		return %bufferheight% ;Success
}

getFontSize(hStdout,Byref fontwidth, ByRef fontheight) {
	
	global sType
	global SizeConErrorLevel
	
	;CONSOLE_FONT_INFO cmdft;
	;GetCurrentConsoleFont(hStdout,FALSE,&cmdft);
	;COORD fontSize = GetConsoleFontSize(hStdout,cmdft.nFont);
	;return fontSize.X;
	
	;typedef struct _CONSOLE_FONT_INFO {
	;	DWORD nFont;
	;	COORD dwFontSize;
	; } CONSOLE_FONT_INFO, *PCONSOLE_FONT_INFO;
	
	VarSetCapacity(struct,sType.DWORD+sType.COORD,0)
	rvalue:=DllCall("GetCurrentConsoleFont","Ptr",hStdout,"Int",0,"Ptr",&struct)
	derror:=ErrorLevel
	;VarSetCapacity(structb,sType.COORD,0)
	;structb:=DllCall("GetConsoleFontSize","Ptr",hStdout,"UInt",NumGet(&struct,"Int"))
	
	fontwidth:=NumGet(&struct,sType.DWORD,"UShort")
	fontheight:=NumGet(&struct,sType.DWORD+sType.SHORT,"UShort")
	
	if (derror) or (rvalue=0)
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("getFontSize",hStdout) ;Failure
	}
	else
		return 0 ;Success
}

getFontWidth(hStdout) {
	
	global SizeConErrorLevel
	
	if (getFontSize(hStdout,fontwidth,fontheight))
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("getFontWidth",hStdout) ;Failure
	}
	else
		return %fontwidth% ;Success
}

getFontHeight(hStdout) {
	
	global SizeConErrorLevel
	
	if (getFontSize(hStdout,fontwidth,fontheight))
	{
		SizeConErrorLevel := ErrorLevel
		return MsgError("getFontHeight",hStdout) ;Failure
	}
	else
		return %fontheight% ;Success
}

;set Console window size
setConsoleSize(width,height,hStdout,hConsole) {
	
	global sType
	global SizeConErrorLevel
	
	getConsoleSize(hStdout,cW,cH) ;buffer size
	WinGetPos,wX,wY,,,ahk_id %hConsole% ;window size
	getFontSize(hStdout,fW,fH) ;font size
	
	;MsgBox % "rqW: " width "`nrqH: " height
	
	newBuffer := Object("w",(width*fW),"h",(height*fH))
	oldBuffer := Object("w",(cW*fW),"h",(cH*fH))
	
	VarSetCapacity(bufferSize,sType.COORD,0)
	NumPut(width,bufferSize,"UShort")
	NumPut(height,bufferSize,sType.SHORT,"UShort")
	
	if ( (newBuffer.w >= oldBuffer.w) and (newBuffer.h >= oldBuffer.h) )
	{
		if (DllCall("SetConsoleScreenBufferSize","Ptr",hStdout,"uint",Numget(bufferSize,"uint"))
			and DllCall("MoveWindow","Ptr",hConsole,"Int",wX,"Int",wY,"Int",newBuffer.w,"Int",newBuffer.h,"Int",1))
			return 1
		else
		{
			SizeConErrorLevel := ErrorLevel
			return MsgError("setConsoleSize",width,height,hStdout,hConsole) ;Failure
		}
	}
	else
	{
		if (DllCall("MoveWindow","Ptr",hConsole,"Int",wX,"Int",wY,"Int",newBuffer.w,"Int",newBuffer.h,"Int",1)
			and DllCall("SetConsoleScreenBufferSize","Ptr",hStdout,"uint",Numget(bufferSize,"uint")))
			return 1
		else
		{
			SizeConErrorLevel := ErrorLevel
			return MsgError("setConsoleSize",width,height,hStdout,hConsole) ;Failure
		}
	}
}

MsgError(fname:="",arg1:="",arg2:="",arg3:="",arg4:="") {
	
	global SizeConDebug
	global SizeConErrorLevel
	;calling function name: msgbox % Exception("",-2).what ; from jethrow
	;http://www.autohotkey.com/board/topic/95002-how-to-nest-functions/#entry598796
	if !IsFunc(fname) ;or fname is space
		fname := Exception("",-2).what
	if !IsFunc(fname) ;try again since sometime it return -2() meaning not found...
		fname := "Undefined"
	if (SizeConDebug)
	{
		MsgBox, 262194, MsgError, %fname%() Failure`nErrorlevel: %SizeConErrorLevel%`n`nWill now Exit.
		IfMsgBox, Abort
			ExitApp
		IfMsgBox, Ignore
			return 0
		IfMsgBox, Retry
		{
			return %fname%(arg1,arg2,arg3,arg4)
		}
	}
	return 0
}

